// keystat.cpp -- Keyboard status and bit field demonstration

#include <stream.hpp>
#include <dos.h>
#include <string.h>
#include <disp.h>

struct keyboard {
   unsigned shiftRight  : 1;     // Keyboard flags
   unsigned shiftLeft   : 1;
   unsigned ctrl        : 1;
   unsigned alt         : 1;
   unsigned scrollLock  : 1;
   unsigned numLock     : 1;
   unsigned capsLock    : 1;
   unsigned insert      : 1;
   unsigned             : 8;     // Not used
};

int cmpkeys(void far* p1, void far* p2);

main()
{
   keyboard far *keys = MK_FP(0x0040, 0x0017);  // Pointer to key state
   keyboard oldkeys;    // Holds copy of last known key state
   int done = 0;        // True (1) when user quits program

   disp_open();         // Initialize display package

   while (!done) {
      disp_move(0, 0);     // Move cursor to upper left corner
      disp_eeop();         // Clear to end of page (clears screen)

// Display states of various keys

      disp_printf("Keyboard State\n");
      disp_printf("\nState bits (press and release):");
      disp_printf("\n <Insert> ....... %d", keys->insert);
      disp_printf("\n <Caps lock> .... %d", keys->capsLock);
      disp_printf("\n <Num lock> ..... %d", keys->numLock);
      disp_printf("\n <Scroll lock> .. %d", keys->scrollLock);
      disp_printf("\n\nShift bits. (press, hold, and release):");
      disp_printf("\n <Alt> .......... %d", keys->alt);
      disp_printf("\n <Ctrl> ......... %d", keys->ctrl);
      disp_printf("\n <Shift left> ... %d", keys->shiftLeft);
      disp_printf("\n <Shift right> .. %d", keys->shiftRight);
      disp_printf("\n\nPress above keys to change state");
      disp_printf("\nPress <Ctrl>-<Alt>-<shiftLeft> to quit");

// Press <Ctrl>-<Alt>-<Shift Left> to quit

      done = (   keys->alt 
              && keys->ctrl 
              && keys->shiftLeft);

      oldkeys = *keys;                    // Save key state in oldkeys
      while (cmpkeys(&oldkeys, keys)) ;   // Pause until state changes
   }
   disp_move(25, 0);    // Move cursor to bottom line
   disp_close();        // Close the display package
}

// Return true (1) if bytes at p1 and p2 are the same
int cmpkeys(void far* p1, void far* p2)
{
   return *(char far *)p1 == *(char far *)p2;
}


// Copyright (c) 1990 by Tom Swan. All rights reserved
// Revision 1.00    Date: 08/28/1990   Time: 03:39 pm




