// elevsim.cpp -- Elevator simulation in C++

#include <stream.hpp>
#include <stdlib.h>
#include <time.h>
#include <disp.h>
#include "building.h"

/* -- Function prototype used only by main() */

void initDisplay(void);

/* -- Global variables. These are the only global variables used by
the simulation. Except for the main building object (theAction), the 
variables hold the statistics displayed on the bottom line. */

unsigned totalPeople;      // Number of people handled
unsigned inBuilding;       // People in building now
unsigned leftBuilding;     // People who left building
unsigned avgWait;          // Average no. people waiting
unsigned avgRide;          // Average no. people in elevators
unsigned tookStair;        // Number people who walked
unsigned totalTime;        // Seconds simulation has run

building theAction;        // Building simulation object

/* -- Note: Enable one of the optional sleep() or msleep() statements
below to slow the simulation on a fast system. */

main()
{
   srand(time(NULL));   // Randomize random-number generator
   initDisplay();       // Initialize the display
   theAction.display(); // Display the elevators and labels

/* -- This while-loop handles the entire simulation. It cycles
while the building object (named theAction) returns true through
its `continues()' function. */

   while (theAction.continues()) {
//      sleep(1);          // 1 real second == 1 simulated second
//      msleep(250);       // 1/4 real second = 1 simulated second
//      msleep(125);       // 1/8 real second = 1 simulated second
      theAction.perform(); // Perform all simulation actions
      theAction.display(); // Update the display
   }

/* -- Perform an orderly exit. */

   disp_move(24, 0);    // Position cursor on last line
   disp_showcursor();   // Make cursor visible
   disp_close();        // Close display package
   exit(0);             // Exit with no errors to report
}

/* -- Initialize the display, showing various labels that remain
unchanged throughout the simulation. Also initialize the Zortech
display package for fast screen writes. */

void initDisplay(void)
{
   int i;

   disp_open();               // Initialize display package
   disp_move(0, 0);           // Move to "home" position
   disp_eeop();               // Clear the display
   disp_hidecursor();         // Make cursor invisible
   disp_move(21, 30);         // Display title line
   disp_puts(
      "Elevator Simulation from Learning C++ by Tom Swan");
   disp_startstand();         // Begin reversed-video display
   disp_move(0, 0);
   disp_puts("Floors    ");
   for (i = 0; i < 10; i++)   // Show reversed video floor
      disp_puts("  0    ");   //  numbers at top of screen
   disp_move(22, 0);
   disp_puts(" Total       In     Left  Avg #   Avg #      ");
   disp_puts("Took  Num  Num   Seconds   Elapsed");
   disp_move(23, 0);
   disp_puts("People     Bldg     Bldg  Waiting Riding    ");
   disp_puts("Stair  Elv  Flr      Left      Time");
   disp_endstand();           // End reversed-video display
   disp_move(1, 0);
   for (i = 0; i < 10; i++)   // Display horzontal divider line
      disp_puts("--------");

/* -- Display elevator cables. These are redrawn by the elevator
class as needed while elevators move. */

   for (int row = 2; row < 18; row++) {
      for (int col = 0; col < MAXELEVS; col++) {
         disp_move(row, 12 + (col * 7));
         disp_putc(':');
      }
   }
}


// Copyright (c) 1990 by Tom Swan. All rights reserved
// Revision 1.00    Date: 09/18/1990   Time: 09:16 am


