// dir.cpp -- directory class

#include <stream.hpp>
#include <direct.h>
#include <dos.h>
#include <string.h>
#include "dir.h"

/* -- Date and time structures, plus pointer types to those 
structures. The module uses these structures to extract the packed
fields in the unsigned date and time integer fields passed to a
fileItem object during its construction. */

struct dateStruct {
   unsigned day : 5;
   unsigned month : 4;
   unsigned year : 7;
};
typedef dateStruct *dsp;

struct timeStruct {
   unsigned seconds : 5;
   unsigned minutes : 6;
   unsigned hours : 5;
};
typedef timeStruct *tsp;

/* -- Constructor for fileItem. Passes the directory file name to the
base strItem constructor, and initializes its own fields. Also
adjusts the time from 24- to 12-hour format. */

fileItem::fileItem(FIND *fp) : (fp->name)
{
   unsigned theHour;

   attribute = fp->attribute;
   time = fp->time;
   date = fp->date;
   size = fp->size;
   theHour = tsp(&time)->hours;
   if (theHour >= 13) {
      ampm = 'p';
      theHour -= 12;
   } else {
      if (theHour == 0) theHour = 12;  // midnight
      ampm = 'a';
   }
   ((tsp)&time)->hours = theHour;
}

/* -- Return string stored by object plus the file size, date, and
time of the most recent update to the file. This function is called
by the selector class even though the fileItem class is declared
after the selector module was compiled--a feature made possible by
declaring the getString function virtual. */

char *fileItem::getString(void)
{
   static char buf[65];
   static char dirStr[] = "<DIR>   ";

/* -- Insert file name, size, date, and time into the static buffer
using the form function to convert coded information into text. Note
the call to the base class (strItem) getString function. This
demonstrates how a derived class can access the base class's public
members. */

   strcpy(buf, form("%-14s%7lu  %2d-%02d-%02d  %2d:%02d%c",
      strItem::getString(),
      size,
      dsp(&date)->month,
      dsp(&date)->day,
      dsp(&date)->year + 80,
      tsp(&time)->hours,
      tsp(&time)->minutes,
      ampm));

/* -- Check if a file is a directory. If so, replace the size of the
file in the newly created string with "<DIR>". */

   if ((attribute & FA_DIREC) != 0)
      memcpy(buf+13, dirStr, 8);
   return buf;
}

/* -- Constructor for directory. Passes the window struct variable,
NULL for the window title (changed later by resetDirectory), and 1
(true) for a pop-up style window to the base class selector
constructor. */

directory::directory(winStruct &ws, char *wildCard, int popup)
   : (ws, NULL, popup)
{
   resetDirectory(wildCard);
}

/* -- Dispose any current list of directory items, then read the
current directory into the list. Set the window's title to the
current working directory (cwd). */

void directory::resetDirectory(char *wildCard)
{
   char *buf;           // Current working directory string
   struct FIND *fp;     // Pointer to DOS search-directory info

/* -- Read directory into a list of fileItems */

   hideWindow();
   disposeList();
   fp = findfirst(wildCard, FA_NORMAL | FA_DIREC);
   while (fp) {
      insertItem(new fileItem(fp));
      fp = findnext();
   }

/* -- Set window title to current directory */

   if ((buf = getcwd(NULL, 0)) != NULL) {
      setTitle(buf);
      free(buf);
   }
}


// Copyright (c) 1990 by Tom Swan. All rights reserved
// Revision 1.00    Date: 10/05/1990   Time: 05:11 pm
