// fsize.cpp -- Show file sizes, dates, times, and total size

#include <stream.hpp>
#include <string.h>
#include <dos.h>

/* -- Function prototypes */

void showFileInfo(FIND *fp);
void adjustTime(FIND *fp, char &ampm);

/* -- Date and time bitfield structures */

struct dateStruct {
   unsigned day : 5;
   unsigned month : 4;
   unsigned year : 7;
};
typedef dateStruct *dsp;

struct timeStruct {
   unsigned seconds : 5;
   unsigned minutes : 6;
   unsigned hours : 5;
};
typedef timeStruct *tsp;

main(int argc, char *argv[])
{
   struct FIND *fp;
   char wildCard[129] = "*.*";
   unsigned long total = 0;
   int count = 0;

   if (argc > 1)
      strcpy(wildCard, argv[1]);
   fp = findfirst(wildCard, 0);
   while (fp) {
      showFileInfo(fp);
      total += fp->size;
      count++;
      fp = findnext();
   }
   if (count == 0)
      cout << "no matching files";
   else {
      cout << dec(total, 21) << "  byte(s) in ";
      cout << count << " file(s)";
   }
}

/* -- Display formatted file entry using information addressed by the
FIND structure pointer fp. */

void showFileInfo(FIND *fp)
{
   static char buf[65];
   static char dirStr[] = "<DIR>   ";
   char ampm;

   adjustTime(fp, ampm);
   strcpy(buf, form("%-14s%7lu  %2d-%02d-%02d  %2d:%02d%c",
      fp->name,
      fp->size,
      dsp(&fp->date)->month,
      dsp(&fp->date)->day,
      dsp(&fp->date)->year + 80,
      tsp(&fp->time)->hours,
      tsp(&fp->time)->minutes,
      ampm));
   if ((fp->attribute & FA_DIREC) != 0)
      memcpy(buf+13, dirStr, 8);
   cout << buf << '\n';
}

/* -- Adjust 24-hour time field in FIND structure addressed by fp
to 12-hour format. Return AM or PM character in ampm. */

void adjustTime(FIND *fp, char &ampm)
{
   unsigned theHour;

   theHour = tsp(&fp->time)->hours;
   if (theHour >= 13) {
      ampm = 'p';
      theHour -= 12;
   } else {
      if (theHour == 0) theHour = 12;  // midnight
      ampm = 'a';
   }
   ((tsp)&fp->time)->hours = theHour;
}


// Copyright (c) 1990 by Tom Swan. All rights reserved
// Revision 1.00    Date: 10/06/1990   Time: 10:38 am
