// read.cpp -- Text file lister

#include <stream.hpp>
#include <direct.h>
#include <string.h>
#include "dir.h"

#define FALSE 0
#define TRUE 1

/* -- Function prototype */

void listFiles(char *wildCard);
int listOneFile(char *fname);

/* -- The main program calls listFiles with a wild card argument such
as *.cpp or ??.bak. Or, if only a single file is listed, it calls
listOneFile, bypassing the directory display. If no arguments are
entered on the command line, the program calls listFiles with the
default *.* wild card string. */

main(int argc, char *argv[])
{
   window::startup();
   if (argc == 2)
      if ( (strchr(argv[1], '*') != NULL) ||
           (strchr(argv[1], '?') != NULL) )
         listFiles(argv[1]);
      else
         listOneFile(argv[1]);
   else
      listFiles("*.*");
   window::shutDown();
   exit(0);
}

void listFiles(char *wildCard)
{
   winStruct dwin = {   // Directory window
      3, 19, 41, 18,    // row, column, width, height
      0x1f,             // text attribute
      0x70,             // border attribute
      0x70,             // highlight attribute
      0                 // type
   };
   winStruct mwin = {   // Main window
      0, 0, 80, 24,     // row, column, width, height
      0x4f,             // text attribute
      0x0f,             // border attribute
      0x70,             // highlight attribute
      3                 // type
   };
   directory *dir;      // Directory window object
   window *mainWin;     // Main display window
   strItem *p;          // Ptr to strItem returned by getSelection
   char buf[13];        // File name returned by getString
   char *t;             // Temporary pointer to char

   buf[12] = 0;         // Make sure strings are terminated
   mainWin = new window(mwin,
      " File Lister from \"Learning C++ by Tom Swan\" ");
   mainWin->showWindow();
   dir = new directory(dwin, wildCard, 0);  // 0 == no popup 
   while ((p = dir->getSelection()) != NULL) {
      strncpy(buf, p->getString(), 12);
      if ((t = strchr(buf, ' ')) != NULL) *t = 0;
      if (!listOneFile(buf)) {
         chdir(buf);
         dir->resetDirectory(wildCard);
      }
   }
   delete dir;
   delete mainWin;
}

int listOneFile(char *fname)
{
   winStruct ws = {
      0, 0, 80, 24,  // row, column, width, height
      0x1f,          // text attribute
      0x70,          // border attribute
      0x70,          // highlight attribute
      3              // type
   };
   FILE *fp;
   char buffer[256];
   selector *listWindow = new selector(ws, fname, 1);
   int n;

   fp = fopen(fname, "r");
   if (!fp) return FALSE;
   while (fgets(buffer, 255, fp) != NULL) {
      n = strlen(buffer);
      if (n > 0) buffer[n - 1] = 0;   // Kill newline at end
      listWindow->insertItem(new strItem(buffer));
   }
   fclose(fp);
   listWindow->getSelection();
   delete listWindow;
   return TRUE;
}


// Copyright (c) 1990 by Tom Swan. All rights reserved
// Revision 1.00    Date: 10/06/1990   Time: 08:48 pm
