// readdb.cpp -- Read records from sample data base

#include <stream.hpp>
#include "sample.h"

#define FATAL 1         // Pass to error to halt
#define NONFATAL 0      // Pass to error to continue

int getRec(FILE *fp, rec &oneRec, long recnum);
void showRec(rec &oneRec, long recnum);
void error(const char *s, int halt);

main()
{
   FILE *fp;
   rec oneRec;
   long recnum = 0;
   char s[80];

   fp = fopen("sample.dat", "rb");
   if (!fp) error("Opening file", FATAL);
   for (;;) {
      cout << "\nEnter record number (-1 to quit): ";
      cin >> s;
      recnum = atol(s);
      if (recnum < 0) break;
      if (getRec(fp, oneRec, recnum))
         showRec(oneRec, recnum);
      else
         error("Reading record", NONFATAL);
   }
   fclose(fp);
}

/* -- Read one record from file fp at specified recnum. Return FALSE
if any errors are detected; otherwise, return TRUE. If FALSE, then
contents of oneRec are not defined. */

int getRec(FILE *fp, rec &oneRec, long recnum)
{
   if (fseek(fp, recnum * sizeof(rec), SEEK_SET) != 0)
      return FALSE;
   if (fread(&oneRec, sizeof(rec), 1, fp) != 1)
      return FALSE;
   return TRUE;
}

/* -- Display record number and contents of one record */

void showRec(rec &oneRec, long recnum)
{
   cout << form("%04ld:%-30s %-2s %+8.2f", 
      recnum, oneRec.name, oneRec.state, oneRec.balance);
}

void error(const char *s, int halt)
{

   cout << "\nERROR: " << s;
   if (halt) exit(1);
}


// Copyright (c) 1990 by Tom Swan. All rights reserved
// Revision 1.00    Date: 10/08/1990   Time: 09:38 am
