// sorttxt.cpp -- Sort lines in a text file

#include <stream.hpp>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLINES 500

/* -- Function prototypes */

void error(const char *message);
void instruct(void);
void readText(const char *fname);
void sortText(void);
void writeText(const char *fname);

/* -- Global variables */

char *strings[MAXLINES];   // Array of pointers to strings
int index;                 // strings[] array index

/* -- Comparison function, called by qsort(). Zortech C++ requires
the linkage directive extern "C" to allow the function to be linked
and called correctly. */

extern "C" int compare(const char **a, const char **b)
{
   return strcmp(*a, *b);
}

main(int argc, char *argv[])
{
   if (argc <= 2) instruct();

   readText(argv[1]);
   sortText();
   writeText(argv[2]);
}

/* -- Display error message and exit */

void error(const char *message)
{
   cout << form("\n\nERROR: %s\n\n", message);
   exit(1);
}

/* -- Display instructions and exit */

void instruct(void)
{
   cout << "\nSORTTXT <infile> <outfile>";
   cout << "\nInstructions: Enter the name of a text file";
   cout << "\nto sort plus the name of a new file to create.";
   cout << "\nThe SORTTXT program will read <infile>, sort its";
   cout << "\nlines alphabetically, and write the results to";
   cout << "\n<outfile>. File size is limited by available";
   cout << "\nfree memory, or to " << MAXLINES << " lines.\n";
   cout << "\nThe original file is not changed provided you enter";
   cout << "\ndifferent file names for <infile> and <outfile>.\n";
   exit(1);
}

/* -- Read lines from a text file, store the text on the heap, and
insert the address of each line into the global strings array of char
pointers. */

void readText(const char *fname)
{
   FILE *fp;
   char buffer[256];
   
   fp = fopen(fname, "r");
   if (!fp) error("Opening file");
   while ((index < MAXLINES) && (fgets(buffer, 255, fp) != NULL))
      if ((strings[index++] = strdup(buffer)) == NULL)
         error("Out of memory");
   fclose(fp);
}

/* -- Using the global strings array of char pointers, sort the
strings in memory. Rearranging the char pointers and not the actual
strings saves time--a LOT of time. */
 
void sortText(void)
{
   cout << "Sorting " << index << " lines\n";
   qsort(&strings, index, sizeof(strings[0]), compare); 
}

/* -- Create a new file and write the sorted text lines to it. */

void writeText(const char *fname)
{
   int i;
   FILE *fp;

   fp = fopen(fname, "w");
   if (!fp) error("Creating file");
   for (i = 0; i < index; i++)
      fputs(strings[i], fp);
   fclose(fp);
}


// Copyright (c) 1990 by Tom Swan. All rights reserved
// Revision 1.00    Date: 10/06/1990   Time: 03:35 pm
